﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AI;

public enum EnemyType
{
    NotEnemy,
    Skitter,  //Destroys the core
    demolition, //Destroys your weapons
    scout //looks for open spots on board, doesn't attack
}

[RequireComponent(typeof(HealthController))]
[RequireComponent(typeof(AnimController))]
public class AIController : MonoBehaviour {
    private const string opposingTag = "Defender";
    public EnemyType type;
    public GameObject target;


    private NavMeshAgent agent;

    private void Assign()
    {
        switch (type)
        {
            case EnemyType.Skitter:
                target = GameObject.FindGameObjectWithTag("Core");
                break;

            case EnemyType.demolition:
                target = FindClosestDefender();
                break;

            default:
                break;
        }

        if (target != null)
            agent.SetDestination(target.transform.position);
    }
	void Start () {
        agent = GetComponent<NavMeshAgent>();
        agent.speed = 1f;
        Invoke("Assign", .1f);
	}

    void Update() {
        //Case for when a weapon hasnt been put down yet
        if(type == EnemyType.demolition) {
            if (target == null || !target.activeSelf)
            {
                //Stand your grinf if nothing to do
                agent.destination = transform.position;

                target = FindClosestDefender();

                //Seek the next target if possible
                if (target != null)
                    agent.SetDestination(target.transform.position);
            }
        }
	}

    private void OnCollisionEnter(Collision collision)
    {
        GameObject obj = collision.gameObject;
        if(obj.CompareTag("Core"))
        {
            obj.GetComponent<CoreManager>().TakeDamage(10);
            gameObject.SetActive(false);
        }
    }

    public bool isMoving()
    {
        return agent.velocity.z != 0; ;
    }

    //Finds closest Opposing Weapon
    private GameObject FindClosestDefender()
    {
        GameObject[] targs = GameObject.FindGameObjectsWithTag(opposingTag);
        GameObject closest = null;
        float distance = Mathf.Infinity;
        Vector3 position = transform.position;

        foreach (GameObject t in targs)
        {
            Vector3 diff = t.transform.position - position;
            float curDistance = diff.sqrMagnitude;

            if (curDistance < distance)
            {
                closest = t;
                distance = curDistance;
                }
            }

        return closest;
        }
    }
