﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;
using UnityEngine.UI;
using Assets.Scripts.Systems;

[RequireComponent(typeof(DefenderCombat))]
public class DefenderBehavior : MonoBehaviour {
    public Image batDisplay;
    private const string dataPath = "Assets/Images/";
    private Sprite activeBat;
    private Sprite inactiveBat;
    public GameObject lightning;
    //
    public float powerRequirement = 25f;
    private bool isOn = false;

    //Colors
    private Color dead;
    private Color alive;

    private const string mTag = "Defender";

	void Start () {
        tag = mTag;

        activeBat = (Sprite) AssetDatabase.LoadAssetAtPath(dataPath + "battery_active.png", typeof(Sprite));
        inactiveBat = (Sprite)AssetDatabase.LoadAssetAtPath(dataPath + "battery_inactive.png", typeof(Sprite));

        setDisplay();

        //Setting colors
        Color colorA = batDisplay.color;
        alive = new Color(colorA.r, colorA.g, colorA.b, 100);
        Color colorD = batDisplay.color;
        dead = new Color(colorD.r, colorD.g, colorD.b, 0);
        //GetComponent<Renderer>().material = GameManager.instance.GetMachineMaterial(false);
        activate();
    }
	
	void Update () {
        if(!CoreManager.coreActive)
        {
            lightning.SetActive(false);
        }

        hideBattery();
	}

    //Active a deactivate defender
    public void activate()
    {
        if (!isOn)
        {
            if (!GameManager.instance.DrainPower(powerRequirement))
                return;
        }
        else
        {
            GameManager.instance.ReturnPower(powerRequirement);
        }
        isOn = !isOn;
        batDisplay.sprite = (isOn) ? activeBat : inactiveBat;
        //GetComponent<Renderer>().material = GameManager.instance.GetMachineMaterial(isOn);
        lightning.SetActive(isOn);
    }

    //Sets up the icon display
    private void setDisplay()
    {
        GameObject world = GameObject.Find("WORLD");
        GameObject mSprite = new GameObject();
        batDisplay = mSprite.AddComponent<Image>();
        mSprite.transform.parent = world.transform;
        mSprite.transform.position = transform.position + Vector3.up * (4.5f);
        batDisplay.sprite = inactiveBat;
        RectTransform rec = batDisplay.GetComponent<RectTransform>();
        rec.eulerAngles = rec.eulerAngles + Vector3.right * 90;
        rec.sizeDelta = new Vector2(1.5f, 1.5f);
    }

    //Fades the battery icon in
    public void showBattey()
    {
        batDisplay.color = Color.Lerp(batDisplay.color, alive, .05f * Time.deltaTime);
    }

    //Fades the battery Icon Out
    private void hideBattery()
    {
        batDisplay.color = Color.Lerp(batDisplay.color, dead, 2.25f * Time.deltaTime);
    }

    //Returns if Defender is on
    public bool ActiveState
    {
        get
        {
            return isOn;
        }
    }
}
